
/* LearningProblem.java.

	$Log: LearningProblem.java,v $
	Revision 1.1  2006/04/26 13:19:18  richard
	Added the file.
	
 
 */


package jsvmstruct;

import java.io.*;

public abstract class LearningProblem<PatternType, LabelType> {
	
	protected StructLearnParam structParams;
	protected StructModel structModel;
	
	public abstract Example<PatternType, LabelType>[] readStructExamples(String filename) throws Throwable;
	
	public abstract void initStructModel(Example<PatternType, LabelType>[] sample) throws Throwable;
	
	public Constraint[] initStructConstraints() throws Throwable {
		return null;
	}
	
	public abstract LabelType classifyStructExample(PatternType x) throws Throwable;
	
	public abstract LabelType findMostViolatedConstraint(PatternType x, LabelType y) throws Throwable;
	
	public abstract FeatureVector psi(PatternType x, LabelType y) throws Throwable;
	
	public double loss(LabelType y, LabelType ybar) throws Throwable {
		return y.equals(ybar)? 0: 1;
	}
	
	public void printStructLearningStats(Example<PatternType, LabelType>[] sample,
			Constraint[] cset, double[] alpha) throws Throwable {
	}
	
	public void writeStructModel(String file) throws Throwable {
		ObjectOutputStream out = new ObjectOutputStream
		(new BufferedOutputStream(new FileOutputStream(file)));
		out.writeObject(structModel);
		out.close();
	}
	
	public StructModel readStructModel(String file) throws Throwable {
		ObjectInputStream in = new ObjectInputStream
		(new BufferedInputStream(new FileInputStream(file)));
		return (StructModel) in.readObject();
	}
	
	public void printStructHelp() {
	}
	
	public void parseStructParameters() throws Throwable {
	}
	
	public void test(String[] argv) throws Throwable {
		Example<PatternType, LabelType>[] exs = readStructExamples(argv[0]);
		structModel = readStructModel(argv[1]);
		PrintWriter out = new PrintWriter(new FileWriter(argv[2]));
		
		int n = 0;
		double total_loss = 0;
		
		for(Example<PatternType, LabelType> ex: exs) {
			LabelType ybar = classifyStructExample(ex.x);
			out.println(ybar);
			total_loss += loss(ex.y, ybar);
			n++;
		}
		
		out.close();
		
		System.out.println("Average loss: " + (total_loss / n));
	}
}
