
/* Multiclass.java.

	$Log: Multiclass.java,v $
	Revision 1.1  2006/04/26 13:23:10  richard
	Added the file.
	

 */

import jsvmstruct.*;

import java.util.*;
import java.io.*;

public class Multiclass extends LearningProblem<FeatureVector, Integer> {
	
	static class MulticlassStructModel extends StructModel {
		private static final long serialVersionUID = 0L;
	    int nclasses;
	    int nvalues;
	}
	
	public Multiclass() {
		structParams = new StructLearnParam();
		structModel = new MulticlassStructModel();
	}
	
	public Example<FeatureVector, Integer>[] readStructExamples(String filename) {
		ArrayList al = new ArrayList();
		try {
			MulticlassStructModel sm = (MulticlassStructModel) structModel;
			
			Scanner sc = new Scanner(new File(filename));
			while(sc.hasNextLine()) {
				String line = sc.nextLine();
				
				int hInd = line.indexOf('#');
				
				String l2 = hInd == -1? line.trim(): 
					line.substring(0, hInd).trim();
				
				String[] tokens = l2.split(" ");
				
				Integer cls = Integer.parseInt(tokens[0]);
				
				if(cls > sm.nclasses)
					sm.nclasses = cls;
				
				FeatureVector v = new FeatureVector();
				v.indices = new int[tokens.length - 1];
				v.values = new double[tokens.length - 1];
				
				for(int i = 1; i < tokens.length; i++) {
					String[] t2 = tokens[i].split(":");
					v.indices[i-1] = Integer.parseInt(t2[0]);
					v.values[i-1] = Double.parseDouble(t2[1]);
					
					if(v.indices[i-1] + 1 > sm.nvalues)
						sm.nvalues = v.indices[i-1] + 1;
				}
				
				Example ex = new Example();
				ex.x = v;
				ex.y = cls;
				
				al.add(ex);
			}
		} catch(Exception e) {
			e.printStackTrace();
			throw new RuntimeException("Exception!");
		}
		
		return (Example<FeatureVector, Integer>[]) al.toArray(new Example[0]);
	}
	
	public void initStructModel(Example<FeatureVector, Integer>[] sample) {
		MulticlassStructModel sm = (MulticlassStructModel) structModel;
		sm.sizePsi = sm.nclasses * sm.nvalues;
	}
	
	public Integer classifyStructExample(FeatureVector x) {
		return findMostViolatedConstraint(x, 0);
	}
	
	public Integer findMostViolatedConstraint(FeatureVector x, Integer y) {

		MulticlassStructModel sm = (MulticlassStructModel) structModel;
		
		FeatureVector fv = (FeatureVector) x;
		
		int worst_class = -1;
		double worst_value = Double.NEGATIVE_INFINITY;
		
		//System.out.println(
		
		for(int i = 1; i <= sm.nclasses; i++) {
			if(y.equals(i))
				continue;
			
			double value = -structModel.b;
			
			int offset = (i - 1) * sm.nvalues;
			
			for(int j = 0; j < fv.indices.length; j++) {
				if(fv.indices[j] > sm.nvalues)
					continue;
				int index = fv.indices[j] + offset;
				value += structModel.w[index] * fv.values[j];
			}
			
			//System.out.println("class = " + i + ", value = " + value);
			
			if(value > worst_value) {
				worst_class = i;
				worst_value = value;
			}
		}
		
		//System.out.println("findMost returns " + worst_class + " ("
		//		   + worst_value + ")");
		
		return new Integer(worst_class);
	}
	
	public FeatureVector psi(FeatureVector x, Integer y) {
		MulticlassStructModel sm = (MulticlassStructModel) structModel;
		
		FeatureVector out = ((FeatureVector) x).copy();
		int offset = sm.nvalues * ((Integer) y - 1) + 0;
		
		for(int i = 0; i < out.indices.length; i++)
			out.indices[i] += offset;
		
		return out;
	}
	
}
